package eta_forum_hub

import (
	"encoding/json"
	"eta/eta_forum_admin/models"
	"eta/eta_forum_admin/models/eta_trial"
	"eta/eta_forum_admin/services/alarm_msg"
	"eta/eta_forum_admin/utils"
	"fmt"
	"io/ioutil"
	"net/http"
	"strings"
	"time"
)

const (
	LoginAuthCodeSource = 1
)

// MiddleServerResultData 中间服务响应体
type MiddleServerResultData struct {
	Code   int         `description:"状态码"`
	Msg    string      `description:"提示信息"`
	Data   interface{} `description:"返回数据"`
	ErrMsg string      `description:"错误信息，不用返回给前端，只是做日志记录"`
}

// AddEtaTrialAdminReq 体验版用户添加手工权限
type AddEtaTrialAdminReq struct {
	UserName    string `description:"客户名称"`
	Password    string
	Account     string
	CompanyName string `description:"客户公司姓名"`
	Position    string `description:"职位"`
	Mobile      string `description:"手机号"`
	Enabled     int    `description:"1:有效，0:禁用"`
	SellerId    int    `description:"销售id"`
	Seller      string `description:"销售员名称"`
	CreateTime  time.Time
	ModifyTime  time.Time
}

type EnabledEtaTrialAdminReq struct {
	Mobile string `description:"手机号"`
}

// AddEtaTrialAdmin CRM_ETA服务-体验版用户添加手工权限
func AddEtaTrialAdmin(req *eta_trial.EtaTrial) (err error) {
	// 体验版无测试环境
	//if utils.RunMode != "release" {
	//	return
	//}
	defer func() {
		if err != nil {
			alarm_msg.SendAlarmMsg(fmt.Sprintf("AddEtaTrialAdmin, 新增试用客户手工录入权限失败, ErrMsg: %s", err.Error()), 3)
		}
	}()
	param := AddEtaTrialAdminReq{
		UserName:    req.UserName,
		Password:    req.Password,
		Account:     req.Account,
		CompanyName: req.CompanyName,
		Position:    req.Position,
		Mobile:      req.Mobile,
		Enabled:     req.Enabled,
		CreateTime:  req.CreateTime,
		ModifyTime:  req.ModifyTime,
	}
	reqJson, e := json.Marshal(param)
	if e != nil {
		err = fmt.Errorf("data json marshal err: %s", e.Error())
		return
	}

	_, resultByte, err := post(string(reqJson), "/v1/eta_trial/admin/add")
	resp := new(models.BaseResponse)
	err = json.Unmarshal(resultByte, &resp)
	if err != nil {
		return
	}
	if resp.Ret != 200 {
		err = fmt.Errorf("result: %s, errmsg: %s", resp.Msg, resp.ErrMsg)
		return
	}
	return
}

// EnabledEtaTrialAdmin CRM_ETA服务-体验版用户添加手工权限
func EnabledEtaTrialAdmin(mobile string) (err error) {
	// 体验版无测试环境
	//if utils.RunMode != "release" {
	//	return
	//}
	defer func() {
		if err != nil {
			alarm_msg.SendAlarmMsg(fmt.Sprintf("EnabledEtaTrialAdmin, 新增试用客户手工录入权限失败, ErrMsg: %s", err.Error()), 3)
		}
	}()
	param := EnabledEtaTrialAdminReq{
		Mobile: mobile,
	}
	reqJson, e := json.Marshal(param)
	if e != nil {
		err = fmt.Errorf("data json marshal err: %s", e.Error())
		return
	}

	_, resultByte, err := post(string(reqJson), "/v1/eta_trial/admin/enabled")
	resp := new(models.BaseResponse)
	err = json.Unmarshal(resultByte, &resp)
	if err != nil {
		return
	}
	if resp.Ret != 200 {
		err = fmt.Errorf("result: %s, errmsg: %s", resp.Msg, resp.ErrMsg)
		return
	}
	return
}

type GetEtaTrialMobileCountReq struct {
	Mobile string `description:"手机号"`
}

type EtaTrialMobileCount struct {
	Count int
}
type GetEtaTrialMobileCountResp struct {
	Ret         int
	Msg         string
	ErrMsg      string
	ErrCode     string
	Data        *EtaTrialMobileCount
	Success     bool `description:"true 执行成功，false 执行失败"`
	IsSendEmail bool `json:"-" description:"true 发送邮件，false 不发送邮件"`
	IsAddLog    bool `json:"-" description:"true 新增操作日志，false 不新增操作日志" `
}

// GetEtaTrialMobileCount CRM_ETA服务-体验版用户添加手工权限
func GetEtaTrialMobileCount(mobile string) (count int, err error) {
	// 体验版无测试环境
	//if utils.RunMode != "release" {
	//	return
	//}
	defer func() {
		if err != nil {
			alarm_msg.SendAlarmMsg(fmt.Sprintf("GetEtaTrialMobileCount, 新增试用客户手工录入权限失败, ErrMsg: %s", err.Error()), 3)
		}
	}()
	param := GetEtaTrialMobileCountReq{
		Mobile: mobile,
	}
	reqJson, e := json.Marshal(param)
	if e != nil {
		err = fmt.Errorf("data json marshal err: %s", e.Error())
		return
	}

	_, resultByte, err := post(string(reqJson), "/v1/eta_trial/mobile/count")
	resp := new(GetEtaTrialMobileCountResp)
	err = json.Unmarshal(resultByte, &resp)
	if err != nil {
		return
	}
	if resp.Ret != 200 {
		err = fmt.Errorf("result: %s, errmsg: %s", resp.Msg, resp.ErrMsg)
		return
	}
	count = resp.Data.Count
	return
}

// post
func post(paramStr string, urlStr string) (resp *models.BaseResponse, result []byte, err error) {
	if utils.ETA_FORUM_HUB_URL == "" {
		err = fmt.Errorf("ETA社区桥接服务地址为空")
		return
	}
	postUrl := utils.ETA_FORUM_HUB_URL + urlStr
	result, err = HttpPost(postUrl, paramStr, "application/json")
	if err != nil {
		err = fmt.Errorf("调用ETA社区桥接服务接口失败 error:%s", err.Error())
		return
	}
	err = json.Unmarshal(result, &resp)
	if err != nil {
		return
	}

	return
}

func HttpPost(url, postData string, params ...string) ([]byte, error) {
	body := ioutil.NopCloser(strings.NewReader(postData))
	client := &http.Client{}
	req, err := http.NewRequest("POST", url, body)
	if err != nil {
		return nil, err
	}
	contentType := "application/x-www-form-urlencoded;charset=utf-8"
	if len(params) > 0 && params[0] != "" {
		contentType = params[0]
	}
	req.Header.Set("Content-Type", contentType)
	req.Header.Set("authorization", utils.MD5(utils.ETA_FORUM_HUB_NAME_EN+utils.ETA_FORUM_HUB_MD5_KEY))
	resp, err := client.Do(req)
	if err != nil {
		return nil, err
	}
	defer resp.Body.Close()
	b, err := ioutil.ReadAll(resp.Body)
	utils.FileLog.Debug("HttpPost:" + string(b))
	return b, err
}
